--  Copyright (C) 2006 Solomon Peachy (pizza@shaftnet.org)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.29 to v2.30 patch
----------------------------------------------------------

-- This function searches for exact matches of 'keyword' in 'string'.
-- The input 'string' is tokenized by a space, and each token is comapred
-- to the 'keyword'. True is returned if the token fully matches the
-- keyword. If the 'keyword' did not match any of the tokens of 'string'
-- false is returned.
-- This function is case sensitive.

create function tokenized_like(varchar, varchar) returns bool as '
declare
  string alias for $1;
  keyword alias for $2;

  token varchar;
  character char;
  string_length integer;
  token_length integer;
  token_begin integer;
  index_cursor integer;
  head_cursor integer;
begin
  head_cursor := 1;
  string_length := char_length(string);

  while head_cursor < string_length loop
    token_length := 0;
    index_cursor := head_cursor;
    token_begin := head_cursor;
    character := substring(string from index_cursor for 1);
    while index_cursor <= string_length loop
      if ((character = \' \') or (character = \';\') or (index_cursor = string_length)) then
        if (index_cursor = string_length) then
          token_length := token_length + 1;
        end if;
        token := substring(string from token_begin for token_length);
        if (token = keyword) then
          return true;
        end if;
      end if;
      token_length := token_length + 1;
      index_cursor := index_cursor + 1;
      character := substring(string from index_cursor for 1);
    end loop;

    index_cursor := head_cursor;
    while index_cursor <= string_length loop
      character := substring(string from index_cursor for 1);
      if ((character = \' \') or (character = \';\') or (index_cursor = string_length)) then
        head_cursor := index_cursor + 1;
        exit;
      end if;
      index_cursor := index_cursor + 1;
    end loop;
  end loop;

  return false;
end;
'
language 'plpgsql';

-- we define operators for each of our functions so that we can easily integrate
-- them into the curret PO search queries
create operator ==== (
  leftarg = varchar,
  rightarg = varchar,
  procedure = tokenized_like,
  commutator = ====
);

-- This function is exactly like tokenized_like, but its first argument is integer
create function tokenized_like(integer, varchar) returns bool as '
declare
  identifier alias for $1;
  keyword alias for $2;
begin
  return tokenized_like(text(identifier), keyword);
end;
'
language 'plpgsql';


-- we define operators for each of our functions so that we can easily integrate
-- them into the curret PO search queries
create operator ==== (
  leftarg = integer,
  rightarg = varchar,
  procedure = tokenized_like,
  commutator = ====
);

-- This function is exactly like tokenized_ilike, but it is case insesnitive
create function tokenized_ilike(varchar, varchar) returns bool as '
declare
  string alias for $1;
  keyword alias for $2;
begin
  return tokenized_like(lower(string), lower(keyword));
end;
'
language 'plpgsql';

-- we define operators for each of our functions so that we can easily integrate
-- them into the curret PO search queries
create operator =##= (
  leftarg = varchar,
  rightarg = varchar,
  procedure = tokenized_ilike,
  commutator = =##=
);

-- This function is exactly like tokenized_like, but its first argument is integer
create function tokenized_ilike(integer, varchar) returns bool as '
declare
  identifier alias for $1;
  keyword alias for $2;
begin
  return tokenized_ilike(text(identifier), keyword);
end;
'
language 'plpgsql';

-- we define operators for each of our functions so that we can easily integrate
-- them into the curret PO search queries
create operator =##= (
  leftarg = integer,
  rightarg = varchar,
  procedure = tokenized_ilike,
  commutator = =##=
); 

-- version update
update version set identifier=3, description='2.30 customer release', major=2, minor=30, variation='B';
