--  Copyright (c) 2007 Solomon Peachy (pizza@shaftnet.org)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.35.B to 2.36.A patch 
----------------------------------------------------------

-- Add a thumbnail to folders and albums

alter table folder add column thumb_ver integer references photo_version(identifier);
alter table album add column thumb_ver integer references photo_version(identifier);

update folder set thumb_ver = (select photo_version.identifier from photo_version, photo where photo.identifier = photo_version.photo and photo.folder = folder.identifier order by date_of_creation limit 1) where thumb_ver is null and caption != 'Trash';
update album set thumb_ver = (select c.version from album_content c where c.album = album.identifier order by c.date_added limit 1) where thumb_ver is null and caption != 'Spool';

-- Add a database function to do the location string matches for us.

CREATE OR REPLACE FUNCTION location_str(integer, character varying) RETURNS character varying AS '
DECLARE 
  l_place character varying;
  l_city character varying;
  l_state character varying;
  l_country character varying;
  l_final character varying[];
  l_return character varying;
BEGIN
  SELECT INTO l_place,l_city,l_state,l_country place,city,state,country FROM view_location WHERE identifier = $1;
  
  IF l_place IS NOT NULL AND char_length(l_place) > 0 THEN SELECT INTO l_final array_append(l_final, l_place); END IF;
  IF l_city IS NOT NULL AND char_length(l_city) > 0 THEN SELECT INTO l_final array_append(l_final, l_city); END IF;
  IF l_state IS NOT NULL  AND char_length(l_state) > 0 THEN SELECT INTO l_final array_append(l_final, l_state); END IF;
  IF l_country IS NOT NULL AND char_length(l_country) > 0 THEN SELECT INTO l_final array_append(l_final, l_country); END IF;

  SELECT INTO l_return array_to_string(l_final, $2);
  RETURN l_return;
END; ' 
LANGUAGE plpgsql;

-- Start merging equipment!

create table equipment_sku (
        identifier integer primary key,
	type integer not null,  -- add a constraint here? 1=lens,2=film, etc
	manufacturer integer references manufacturer(identifier) not null,	
	model text not null,
        properties text,
        url text
);

create sequence equipment_sku_id_seq;

create table equipment (
	identifier integer primary key,
	sku integer references equipment_sku(identifier) not null,
	users integer references users(identifier) not null,
	purchase_date timestamp,
	purchased_new boolean not null,
	access_rights integer references access_type(identifier) not null,
        properties text,
	serial text
);

create sequence equipment_id_seq;

create table equipment_types (
       identifier integer primary key,
       name text not null
);

insert into equipment_types values (1, 'filter');
insert into equipment_types values (2, 'flash');
insert into equipment_types values (3, 'scanner');
insert into equipment_types values (4, 'support');
insert into equipment_types values (5, 'lens');

create table photo_equipment (
	photo integer references photo(identifier) not null,
	equipment integer references equipment(identifier) not null,
	type integer references equipment_types(identifier) not null
);

create index photo_equipment_photo_idx on photo_equipment(photo);

CREATE VIEW view_equipment AS
    SELECT e.identifier, e.sku, e.users, e.purchase_date, e.purchased_new, e.access_rights, e.properties, e.serial,
           s.type, s.model, s.properties as s_properties, s.url as s_url, s.manufacturer as manu_id,
           m.name, m.url as m_url,
           t.name as type_name
      FROM equipment e, equipment_sku s, manufacturer m, equipment_types t
     WHERE e.sku = s.identifier 
       AND s.manufacturer = m.identifier
       AND t.identifier = s.type
  ORDER BY m.name, s.model;

CREATE VIEW view_equipment_sku AS
    SELECT s.identifier, s.type, s.model, s.properties as s_properties, s.url as s_url, s.manufacturer as manu_id,
           m.name, m.url as m_url,
           t.name as type_name
      FROM equipment_sku s, manufacturer m, equipment_types t
     WHERE s.manufacturer = m.identifier
       AND t.identifier = s.type
  ORDER BY m.name, s.model;

CREATE OR REPLACE FUNCTION number_of_equipment_references(integer) RETURNS bigint AS 'select count(photo) from photo_equipment where equipment = $1' LANGUAGE sql;

-- version update 
update version set identifier=3, description='2.36 customer release', 
 major=2, minor=36, variation='A';


